﻿//
// Copyright (c) 2014, MindFusion LLC - Bulgaria.
//

using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Data.SqlServerCe;
using System.Configuration;

namespace MindFusion.Scheduling.WebForms.Samples.CS.Database
{
	public partial class Default : System.Web.UI.Page
	{
		protected void Page_Load(object sender, EventArgs e)
		{
			if (!IsPostBack)
			{
				// Load the schedule data from the database
				LoadFromDB();
			}
		}

		private void LoadFromDB()
		{
			Calendar1.Schedule.Clear();

			using (SqlCeConnection conn = new SqlCeConnection(ConfigurationManager.ConnectionStrings["PlannerConnString"].ConnectionString))
			{
				conn.Open();

				// Load contacts
				using (SqlCeCommand cmd = new SqlCeCommand
					(
						"SELECT * FROM Contacts", conn
					))
				{
					using (SqlCeDataReader reader = cmd.ExecuteReader())
					{
						while (reader.Read())
						{
							Contact c = new Contact();
							c.Tag = reader[0]; // ID
							c.FirstName = reader.GetString(1);
							c.LastName = reader.GetString(2);

							Calendar1.Contacts.Add(c);
						}
					}
				}

				// Load appointmets
				using (SqlCeCommand cmd = new SqlCeCommand
					(
						"SELECT * FROM Events", conn
					))
				{
					using (SqlCeDataReader reader = cmd.ExecuteReader())
					{
						while (reader.Read())
						{
							Appointment a = new Appointment();
							a.Tag = reader[0]; // ID
							a.HeaderText = reader.GetString(1);
							a.StartTime = reader.GetDateTime(2);
							a.EndTime = reader.GetDateTime(3);

							Calendar1.Schedule.Items.Add(a);
						}
					}
				}

				// Assign appointments to contacts
				foreach (Appointment a in Calendar1.Schedule.Items)
				{
					// Select all contacts for the given appointment
					using (SqlCeCommand cmd = new SqlCeCommand
						(
							"SELECT [ContactID] FROM ContactEvents WHERE [EventID] = @EVTID", conn
						))
					{
						cmd.Parameters.AddWithValue("EVTID", a.Tag);
						using (SqlCeDataReader reader = cmd.ExecuteReader())
						{
							while (reader.Read())
							{
								int contactTag = (int)reader[0];
								Contact con = Calendar1.Schedule.Contacts.Where(c => (int)c.Tag == contactTag).FirstOrDefault();
								if (con != null)
									a.Contacts.Add(con);
							}
						}
					}
				}
			}
		}

		private void CreateAppointment(Appointment app)
		{
			using (SqlCeConnection conn = new SqlCeConnection(ConfigurationManager.ConnectionStrings["PlannerConnString"].ConnectionString))
			{
				conn.Open();

				using (SqlCeCommand cmd = new SqlCeCommand
					(
						"INSERT INTO Events ([HeaderText], [StartTime], [EndTime]) " +
						"VALUES (@EVTTEXT, @EVTSTIME, @EVTETIME); ", conn
					))
				{
					cmd.Parameters.AddWithValue("EVTTEXT", app.HeaderText);
					cmd.Parameters.AddWithValue("EVTSTIME", app.StartTime);
					cmd.Parameters.AddWithValue("EVTETIME", app.EndTime);

					cmd.ExecuteNonQuery();

					cmd.CommandText = "SELECT @@IDENTITY as NewId";
					app.Tag = (int)(decimal)cmd.ExecuteScalar();
				}

				foreach (Contact con in app.Contacts)
				{
					using (SqlCeCommand cmd = new SqlCeCommand
						(
							"INSERT INTO ContactEvents ([EventID], [ContactID]) " +
							"VALUES (@EVTID, @CNTID); ", conn
						))
					{
						cmd.Parameters.AddWithValue("EVTID", app.Tag);
						cmd.Parameters.AddWithValue("CNTID", con.Tag);

						cmd.ExecuteNonQuery();
					}
				}
			}
		}

		private void DeleteAppointment(Appointment app)
		{
			using (SqlCeConnection conn = new SqlCeConnection(ConfigurationManager.ConnectionStrings["PlannerConnString"].ConnectionString))
			{
				conn.Open();

				using (SqlCeCommand cmd = new SqlCeCommand
					(
						"DELETE FROM ContactEvents " +
						"WHERE [EventID] = @EVTID ", conn
					))
				{
					cmd.Parameters.AddWithValue("EVTID", app.Tag);
					cmd.ExecuteNonQuery();

					cmd.CommandText =
						"DELETE FROM Events " +
						"WHERE [ID] = @EVTID ";
					cmd.ExecuteNonQuery();
				}
			}
		}

		private void EditAppointment(Appointment app)
		{
			using (SqlCeConnection conn = new SqlCeConnection(ConfigurationManager.ConnectionStrings["PlannerConnString"].ConnectionString))
			{
				conn.Open();

				using (SqlCeCommand cmd = new SqlCeCommand
					(
						"UPDATE Events SET " +
						"[HeaderText] = @EVTTEXT, [StartTime] = @EVTSTIME, [EndTime] = @EVTETIME " +
						"WHERE [ID] = @EVTID", conn
					))
				{
					cmd.Parameters.AddWithValue("EVTTEXT", app.HeaderText);
					cmd.Parameters.AddWithValue("EVTSTIME", app.StartTime);
					cmd.Parameters.AddWithValue("EVTETIME", app.EndTime);
					cmd.Parameters.AddWithValue("EVTID", app.Tag);

					cmd.ExecuteNonQuery();
				}
			}
		}

		protected void Calendar1_ItemCreated(object sender, ItemEventArgs e)
		{
			CreateAppointment((Appointment)e.Item);
		}

		protected void Calendar1_ItemDeleted(object sender, ItemEventArgs e)
		{
			DeleteAppointment((Appointment)e.Item);
		}

		protected void Calendar1_ItemInplaceEdited(object sender, ItemEventArgs e)
		{
			EditAppointment((Appointment)e.Item);
		}

		protected void Calendar1_ItemModified(object sender, ItemModifiedEventArgs e)
		{
			EditAppointment((Appointment)e.Item);
		}

		protected void btnRefresh_Click(object sender, EventArgs e)
		{
			GridView1.DataBind();
		}
	}
}